#include "Creature.h"
#include <string>
#include "AI_Behaviors.h"
#include "Sprite_Animation.h"
#include <stdlib.h>     /* srand, rand */
#include <time.h>       /* time */
#include "State.h"
#include "Wander.h"
#pragma once


	AIMonster::AIMonster()
	{
        mDirection = Ogre::Vector3::ZERO;
		moveValue  = 120;
		scale      = 0.8;
		attackAN   = false;
		idleAN     = false;
		walkAN     = false;
	}//end of constructor


	AIMonster::~AIMonster()
	{
		if(raySceneQuery)
		  sceneMgr->destroyQuery(raySceneQuery);
	}//end of destructor


	void AIMonster::set_moveSpeed(Ogre::Real change)
	{
		moveValue = change;
	}//end of Set_Movespeed

	void AIMonster::set_Scale(Ogre::Real change)
	{
		scale = change;
	}//end of set_Scale

	void AIMonster::set_Name(string Name)
	{
		name = Name;
	}//end of set_meshtype

	void AIMonster::setDestination(Ogre::Vector3 GoingTo)
	{
		mDestination = GoingTo;
	}//end of setDestination


	void AIMonster::Find_Player_Location(Ogre::Vector3 location)
	{
		this->Player_Location = location;
	}//end of Find_Player_Location


	void AIMonster::Set_Animation_Names(string attack, string walk, string idle)
	{
		ATTACK	= attack;
		IDLE	= idle;
		WALK	= walk;
	}//end of Set_Animation_Names
//*****************************************************************************

	void AIMonster::create(Ogre::SceneManager* sentSceneMgr, Ogre::String theName)
	{
		sceneMgr = sentSceneMgr; // get a copy of sceneManager.
		//prePosition = Ogre::Vector3(99,99,99);
		AIB = new AI_Behaviors;
		mDistance = 999999;
		//this->realHealthStt = 9;

		// Create the entity
		entity = sceneMgr->createEntity(theName, "ninja.mesh");
		box = entity->getBoundingBox();
		
		entityNode = sentSceneMgr->getRootSceneNode()->createChildSceneNode();
		entityNode->attachObject(entity); // attach player's entity to player node
		
		entityNode->setScale(scale,scale,scale); // make player's mesh smaller.
		this->prePosition = entityNode->getPosition();

		length = box.getSize().x;
		entityNode->showBoundingBox(true);
		raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());

		this->Current_State = Wander::Get_instance(); 
		this->Current_State->Enter_State(this);
	}//end of create()


	void AIMonster::rotateRobotToDirection(void)
	{
		Ogre::Vector3 src = entityNode->getOrientation() * Ogre::Vector3::UNIT_X;
		src.y = 0;                                                   
		mDirection.y = 0;
		src.normalise();
		Ogre::Real mDistance = mDirection.normalise( );     
		Ogre::Quaternion quat = src.getRotationTo(mDirection);	
		Ogre::Quaternion quat2(Ogre::Degree(-90), Ogre::Vector3::UNIT_X);
		quat * quat2;
		entityNode->rotate(quat);
	}//end of rotateRobotToDirection()


	void AIMonster::update(const Ogre::FrameEvent &evt)
	{
		prePosition = entityNode->getPosition();
		translate(evt);		//move ninja
		
		//if(!isCollision(entityNode->getPosition(),/*mDirection*/entityNode->getOrientation()*Ogre::Vector3(0,0,1)))
		//{
		//	prePosition = entityNode->getPosition();
		//}//end of if
		//else
		//{
		//	entityNode->setPosition(prePosition);
		//}//end of else	
	}//end of update()


	


	void AIMonster::translate(const Ogre::FrameEvent& evt)
	{
		this->Current_State->Execute_State(this);
		this->Current_State->Exit_State(this);
			
		rotateRobotToDirection();
		Ogre::Real move = moveValue * evt.timeSinceLastFrame;
		mDistance -= move;


		if(attackAN)
		{
			Animation(evt,ATTACK);
		}//end of if
		else if(walkAN)
		{
			Animation(evt,WALK);
			entityNode->translate(mDirection * move, 
				Ogre::Node::TS_WORLD);
		}//end of else
		else if(idleAN)
		{
			Animation(evt,IDLE);
		}//end of else
	}//end of translate


	void AIMonster::updateRealStats(){}


	void AIMonster::reset()
	{
	}






	AIMonster_Collection::AIMonster_Collection()
	{
	}//end of constructor


	void AIMonster_Collection::Group_Location(Ogre::Vector3 location)
	{
		GroupLocation = location;
	}//end of GroupLocation


	void AIMonster_Collection::create(Ogre::SceneManager* sentSceneMgr)
	{	

		monsters[0].create(sentSceneMgr, monsters[0].name);
				monsters[0].entityNode->setPosition(GroupLocation.x-45,
				20-monsters[0].box.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*monsters[0].scale,
				GroupLocation.z+45);
		monsters[0].prePosition = monsters[0].entityNode->getPosition();

		monsters[1].create(sentSceneMgr, monsters[1].name);
				monsters[1].entityNode->setPosition(GroupLocation.x+45,
				20-monsters[1].box.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*monsters[1].scale,
				GroupLocation.z-45);
		monsters[1].prePosition = monsters[1].entityNode->getPosition();
		monsters[2].create(sentSceneMgr, monsters[2].name);
				monsters[2].entityNode->setPosition(GroupLocation.x+45,
				20-monsters[2].box.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*monsters[2].scale,
				GroupLocation.z+45);
		monsters[2].prePosition = monsters[2].entityNode->getPosition();
		monsters[3].create(sentSceneMgr, monsters[3].name);
				monsters[3].entityNode->setPosition(GroupLocation.x-45,
				20-monsters[3].box.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*monsters[3].scale,
				GroupLocation.z-45);
		monsters[3].prePosition = monsters[3].entityNode->getPosition();
	}//end of create


	void AIMonster_Collection::update(const Ogre::FrameEvent& evt)
	{
		for(int i =0;i<MAXMONSTER;i++)
		{
			monsters[i].update(evt);

			for(int j = 0; j<4; j++)
			{
				if(i==j)
					continue;
				if(monsters[i].isCollision(monsters[i].entityNode->getPosition(),
					Ogre::Vector3::NEGATIVE_UNIT_Y/*monsters[i].entityNode->getOrientation()*Ogre::Vector3(0,0,2)*/,
					monsters[j].name))
				{
					monsters[i].entityNode->setPosition(monsters[i].prePosition);
					Ogre::LogManager::getSingleton().logMessage("\n\nNinja collision!\n\n");
				}
				//if(monsters[i].isCollision(monsters[i].getPosition(),  
				//	monsters, 4, 
				//	monsters[i].entityNode->getOrientation()*Ogre::Vector3(0,0,2)))
				//{
				//	monsters[i].entityNode->setPosition(monsters[i].prePosition);
				//}
				
			}
			////for(int k = i;k<i;k++)
			////{
			//	for(int j=0;j<4;j++)
			//	{
			//		if(monsters[j].entityNode->getName()!=monsters[i].entityNode->getName())
			//		{
			//			Ogre::Vector3 direction = monsters[j].entityNode->getPosition() - monsters[i].entityNode->getPosition();
			//			Ogre::Real distance	= direction.normalise();

			//			if(distance <= monsters[j].length*2.5)
			//			{
			//				monsters[j].entityNode->setPosition(monsters[j].prePosition);
			//				/*monsters[0].entityNode->setPosition(monsters[0].prePosition*monsters[0].length);
			//				monsters[1].entityNode->setPosition(monsters[1].prePosition*monsters[1].length);
			//				monsters[2].entityNode->setPosition(Ogre::Vector3(300,0,100));
			//				monsters[3].entityNode->setPosition(Ogre::Vector3(1000,0,100));*/
			//				//monsters[i].entityNode->setPosition(this->monsters[i].prePosition);
			//			}//end of if
			//			else
			//				monsters[j].prePosition = monsters[j].entityNode->getPosition();
			//		}
			//	}//end of else
			////}
			
		}//end of for loop

	}//end of update()
