// Title       : Arrow.cpp
// Date        : 21/05/13
// Description : The implmentation of the class for an arrow

#include "Arrow.h"

// ****************************************************************************
// Constructors and Destructors

Arrow::Arrow()
{ // Constructor

  // Initialise the arrow variables
  name = "";
  scale = 8.0;
  shootDistance = 0;
  flightTimer = MAXFLIGHTTIME;
  active = false;
}

Arrow::~Arrow()
{ // Destructor
  if(raySceneQuery)
  { // Destroy the ray scene query
    sceneMgr->destroyQuery(raySceneQuery);
  }
}

// ****************************************************************************
// Getters and Setters

void Arrow::setPosition(Ogre::Vector3 thePosition)
{
  entityNode->setPosition(thePosition);
}

void Arrow::setActive(bool value)
{
  active = value;
}

// ****************************************************************************
// Create and Reset

void Arrow::create(Ogre::SceneManager* sentSceneMgr, Ogre::String theName)
{
  sceneMgr = sentSceneMgr; // get a copy of sceneManager.

  name = theName;

  // Create the entity
  entity = sceneMgr->createEntity(name, "Arrow 01.mesh");
  meshBox = entity->getBoundingBox();
 
  entityNode = sentSceneMgr->getRootSceneNode()->createChildSceneNode();
  entityNode->attachObject(entity); // attach player's entity to player node.
		
  entityNode->setScale(scale,scale,scale); // make player's mesh smaller.
  entityNode->setPosition(-100,-meshBox.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-400); // set position of player node.
  length = meshBox.getSize().x;

  raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());
		
  if(active)
  {
    entityNode->showBoundingBox(true);
    entity->setQueryFlags(true); // turn on/off query for this entity used for collision detection.
	entity->setVisibilityFlags(true); // turn visiblility on/off for this entity.
  }
  else
  {
    entityNode->showBoundingBox(false);
	entity->setQueryFlags(false); // turn on/off query for this entity used for collision detection.
	entity->setVisibilityFlags(false); // turn visiblility on/off for this entity.
  }
}

void Arrow::reset()
{
  flightTimer = MAXFLIGHTTIME;
  shootDistance = 0;
  active = false;
}

// ****************************************************************************
// Update

void Arrow::update(const Ogre::FrameEvent& evt)
{
  if (active)
  {
    flightTimer = flightTimer - 1*evt.timeSinceLastEvent;
	shootDistance-= 14* evt.timeSinceLastFrame;
	entityNode->translate(Ogre::Vector3(0,shootDistance,0),Ogre::Node::TS_LOCAL);

	if (flightTimer <= 0)
	{
      reset();
	}

	entityNode->showBoundingBox(true);
	entity->setQueryFlags(true); // turn on/off query for this entity used for collision detection.
	entity->setVisibilityFlags(true); // turn visiblility on/off for this entity.

	if(isCollisionMonster(entityNode->getPosition(),Ogre::Vector3(0,0,0)))
	{
	  sceneMgr->getEntity(this->getCollidedObject())->setVisible(false);
	}
  }
  else
  {
    entityNode->showBoundingBox(false);
	entity->setQueryFlags(false); // turn on/off query for this entity used for collision detection.
	entity->setVisibilityFlags(false); // turn visiblility on/off for this entity.
  }
}

// ****************************************************************************