// Filename: BuildMap.cpp //

#include "BuildMap.h"

BuildMap::BuildMap()
{

}

BuildMap::~BuildMap()
{
}

void BuildMap::createMainFloorPlane(Ogre::SceneManager* sentSceneMgr)
{
	sceneMgr = sentSceneMgr; // get a copy of the sceneMAnager

	// create ray query for collision detection.
	raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());

	//creates a tunnel plane from mesh that can be used for
	//walls, foors and ceilings
    Ogre::Plane plane(Ogre::Vector3::UNIT_Y, 0);

	//create mesh
	Ogre::MeshManager::getSingleton().createPlane("mainFloorPlane", 
		Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
		plane, 15000, 15000, 100, 100, true, 1, 5, 5, Ogre::Vector3::UNIT_Z);

	entity = sceneMgr->createEntity("CaveSystemFloor", "mainFloorPlane");
	entityNode = sceneMgr->getRootSceneNode()->createChildSceneNode();
	entityNode->attachObject(entity );

	//set positions and texture
	entity ->setMaterialName("Examples/RustySteel");
	entity ->setCastShadows(true);

	entityNode->setPosition(0, 0, 0);

	//show Bounding box
	entityNode->showBoundingBox(true);
	
}

void BuildMap::createTunnelPlane(Ogre::SceneManager* sentSceneMgr, Ogre::String planeName,
	 Ogre::Radian angle, Ogre::Real xPos, Ogre::Real yPos, Ogre::Real zPos)
{
	sceneMgr = sentSceneMgr; // get a copy of the sceneMAnager

	// create ray query for collision detection.
	raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());

	//creates a tunnel plane from mesh that can be used for
	//walls, foors and ceilings
	Ogre::Plane plane(Ogre::Vector3::UNIT_Y, 0);

	//create mesh
	Ogre::MeshManager::getSingleton().createPlane("tunnelPlane", 
		Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
		plane, 500, 3000, 20,20, true, 1, 5, 5, Ogre::Vector3::UNIT_Z);

	entity = sceneMgr->createEntity(planeName, "tunnelPlane");
	entityNode = sceneMgr->getRootSceneNode()->createChildSceneNode();
	entityNode->attachObject(entity);

	//set texture
	entity->setMaterialName("Examples/RustySteel");
	entity->setCastShadows(true);


	//set position
	entityNode->setPosition(xPos, yPos, zPos);

	//rotate if needed  (for walls)
    entityNode->roll(angle, Ogre::Node::TS_LOCAL);

	//show Bounding box
	entityNode->showBoundingBox(true);
}

void BuildMap::createCavernPlane(Ogre::SceneManager* sentSceneMgr, Ogre::String planeName,
	  Ogre::Vector3 axis, Ogre::Radian angle, Ogre::Real xPos, Ogre::Real yPos, Ogre::Real zPos)
{
	sceneMgr = sentSceneMgr; // get a copy of the sceneMAnager

	// create ray query for collision detection.
	raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());

	//creates a tunnel plane from mesh that can be used for
	//walls, foors and ceilings
	Ogre::Plane plane(Ogre::Vector3::UNIT_Y, 0);

	//create mesh
	Ogre::MeshManager::getSingleton().createPlane("cavernPlane", 
		Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
		plane, 3000, 3000, 20,20, true, 1, 5, 5, Ogre::Vector3::UNIT_Z);

	entity = sceneMgr->createEntity(planeName, "cavernPlane");
	entityNode = sceneMgr->getRootSceneNode()->createChildSceneNode();
	entityNode->attachObject(entity);

	//set texture
	entity->setMaterialName("Examples/RustySteel");
	entity->setCastShadows(true);

	//set position
	entityNode->setPosition(xPos, yPos, zPos);

	//rotate if needed  (for walls)
	entityNode->rotate(axis, angle, Ogre::Node::TS_LOCAL);

	//show Bounding box
	entityNode->showBoundingBox(true);
}

void BuildMap::createCavernDoorPlaneLeft(Ogre::SceneManager* sentSceneMgr, Ogre::String planeName,
	Ogre::Vector3 axis, Ogre::Radian angle, 
	Ogre::Real xPos, Ogre::Real yPos, Ogre::Real zPos)
{

	sceneMgr = sentSceneMgr; // get a copy of the sceneMAnager

	// create ray query for collision detection.
	raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());

	//creates a tunnel plane from mesh that can be usswed for
	//walls, foors and ceilings
	Ogre::Plane plane(Ogre::Vector3::UNIT_Y, 0);


	//create mesh
	Ogre::MeshManager::getSingleton().createPlane("cavePlaneLeft", 
		Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
		plane, 1500, 3000, 20,20, true, 1, 5, 5, Ogre::Vector3::UNIT_Z);


	entity = sceneMgr->createEntity(planeName, "cavePlaneLeft");
	entityNode = sceneMgr->getRootSceneNode()->createChildSceneNode();
	entityNode->attachObject(entity);


	//set texture
	entity->setMaterialName("Examples/RustySteel");
	entity->setCastShadows(true);

	

	//set position
	entityNode->setPosition(xPos, yPos, zPos);

	//rotate if needed  (for walls)
	entityNode->rotate(axis, angle, Ogre::Node::TS_LOCAL);

	//show Bounding box
	entityNode->showBoundingBox(true);
}

void BuildMap::createCavernDoorPlaneMiddle(Ogre::SceneManager* sentSceneMgr, Ogre::String planeName,
	Ogre::Vector3 axis, Ogre::Radian angle, Ogre::Real xPos, Ogre::Real yPos, Ogre::Real zPos)
{
	sceneMgr = sentSceneMgr; // get a copy of the sceneMAnager

	//creates a tunnel plane from mesh that can be used for
	//walls, foors and ceilings
	Ogre::Plane plane(Ogre::Vector3::UNIT_Y, 0);


	//create mesh
	Ogre::MeshManager::getSingleton().createPlane("cavePlaneMiddle", 
		Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
		plane, 500, 2500, 20,20, true, 1, 5, 5, Ogre::Vector3::UNIT_Z);


	entity = sceneMgr->createEntity(planeName, "cavePlaneMiddle");
	entityNode = sceneMgr->getRootSceneNode()->createChildSceneNode();
	entityNode->attachObject(entity);


	//set texture
	entity->setMaterialName("Examples/RustySteel");
	entity->setCastShadows(true);

	

	//set position
	entityNode->setPosition(xPos, yPos, zPos);

	//rotate if needed  (for walls)
	entityNode->rotate(axis, angle, Ogre::Node::TS_LOCAL);

	//show Bounding box
	entityNode->showBoundingBox(true);
}

void BuildMap::createCavernDoorPlaneRight(Ogre::SceneManager* sentSceneMgr, Ogre::String planeName,
	Ogre::Vector3 axis, Ogre::Radian angle, Ogre::Real xPos, Ogre::Real yPos, Ogre::Real zPos)
{
	sceneMgr = sentSceneMgr; // get a copy of the sceneMAnager

	// create ray query for collision detection.
	raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());

	//creates a tunnel plane from mesh that can be used for
	//walls, foors and ceilings
	Ogre::Plane plane(Ogre::Vector3::UNIT_Y, 0);


	//create mesh
	Ogre::MeshManager::getSingleton().createPlane("cavePlaneRight", 
		Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
		plane, 1000, 3000, 20,20, true, 1, 5, 5, Ogre::Vector3::UNIT_Z);

	entity = sceneMgr->createEntity(planeName, "cavePlaneRight");
	entityNode = sceneMgr->getRootSceneNode()->createChildSceneNode();
	entityNode->attachObject(entity);

	//set texture
	entity->setMaterialName("Examples/RustySteel");
	entity->setCastShadows(true);

	

	//set position
	entityNode->setPosition(xPos, yPos, zPos);

	//rotate if needed  (for walls)
	entityNode->rotate(axis, angle, Ogre::Node::TS_LOCAL);

	//show Bounding box
	entityNode->showBoundingBox(true);
}

void BuildMap::create(Ogre::SceneManager* sentSceneMgr, Ogre::String theName)
{
}

void BuildMap::reset()
{
}

void BuildMap::addGravity(const Ogre::FrameEvent& evt)
{
}