#include "ConfigLoader.h"
#include <OgreResourceGroupManager.h>
#include <OgreStringConverter.h>
#include <OgreException.h>


ConfigLoader::ConfigLoader()
{
	//load the configuration file
	configurationFile.load("GameCaseConfigurations.txt", 
		Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
		"=", true);

	//get a section iterator and declare strings for differing parts
	//of the configuration file
	Ogre::ConfigFile::SectionIterator secI = configurationFile.getSectionIterator();
	Ogre::String sectionName;
	Ogre::String keyName;
	Ogre::String valueName;

	//while the section iterator has more elements get the next key
	//and assign the keyName and valueName and insert into map for use
	while(secI.hasMoreElements())
	{
		sectionName = secI.peekNextKey();
		Ogre::ConfigFile::SettingsMultiMap *settings = secI.getNext();
		Ogre::ConfigFile::SettingsMultiMap::iterator i;
		for(i = settings->begin(); i != settings->end(); i++)
		{
			keyName = i->first;
			valueName = i->second;
			configurations.insert(std::pair<std::string, std::string>
				(sectionName + "/" + keyName, valueName));
		}//end for
	}//end while
}//end constructor

//destructor
ConfigLoader::~ConfigLoader()
{
}//end destructor

//get an instance of COnfigLoader and return it
ConfigLoader ConfigLoader::getInstance()
{
	static ConfigLoader instance;
	return instance;
}//end getInstance

//check to see if a key exists
bool ConfigLoader::getKeyExists(std::string key)
{
	if(configurations.count(key) > 0)
	{
		return true;
	}
	return false;
}//end getKeyExists

//get a boolean value from external file
bool ConfigLoader::getValueAsBool(std::string key)
{
	return Ogre::StringConverter::parseBool(getValueAsString(key));
}//end getValueAsBool

//get an int value from external file
int ConfigLoader::getValueAsInt(std::string key)
{
	return atoi(getValueAsString(key).c_str());
}//end getValueAsInt

//gets value as a string from external file
std::string ConfigLoader::getValueAsString(std::string key)
{
	if(getKeyExists(key) == true)
	{
		return configurations[key];
	}//end if
	else
	{
		throw Ogre::Exception(Ogre::Exception::ERR_ITEM_NOT_FOUND, 
			"Configuration key: " +key + " not found", "ConfigLoader::getValue");
	}//end else
}//end getValueAsString

//gets value as Ogre::Real from external file
Ogre::Real ConfigLoader::getValueAsReal(std::string key)
{
	return Ogre::StringConverter::parseReal(getValueAsString(key));
}//end getValueAsReal

//gets a value as an Ogre::Vector3 from external file
Ogre::Vector3 ConfigLoader::getValueAsVector3(std::string key)
{
	return Ogre::StringConverter::parseVector3(getValueAsString(key));
}//end getValueAsVector3

//gets value as colour value from external file
Ogre::ColourValue ConfigLoader::getValueAsColourValue(std::string key)
{
	return Ogre::StringConverter::parseColourValue(getValueAsString(key));
}//end getValueAsColourValue