// Title       : DateArmsArmour.cpp
// Date        : 21/05/13
// Description : The class for armour that goes on the arms

#include "DateArmsArmour.h"

// ****************************************************************************
// Constructors and Destructors

DateArmsArmour::DateArmsArmour()
{ // Constructor

  // Initialises the arm armour attributes
  name = "";
  scale = 10.0;
  active = true;
  setArmourType(Arms);
  weaponVectorFromRightHand = Ogre::Vector3(0,-60,0);
  objectType = ObjectArmourType;
}

DateArmsArmour::~DateArmsArmour()
{ // Destrcutor

  if(raySceneQuery)
  { // Destroy the ray scene query
    sceneMgr->destroyQuery(raySceneQuery);
  }
}

// ****************************************************************************
// Getters and Setters
void DateArmsArmour::setPosition(Ogre::Vector3 thePosition)
{ // Set the position of the armour
  entityNode->setPosition(thePosition);
}

void DateArmsArmour::setActive(bool value)
{ // Set armour active / inactive
  active = value;
}

// ****************************************************************************
// Create and Reset

void DateArmsArmour::create(Ogre::SceneManager* sentSceneMgr, Ogre::String theName)
{ // Creates the armour

  // Get a copy of the scene manager
  sceneMgr = sentSceneMgr; // get a copy of sceneManager.

  // Set the name of the armour
  name = theName;

  // Create the entity
  entity = sceneMgr->createEntity(name, "Date Model Arms 04 - 03.mesh");
  Ogre::AxisAlignedBox box = entity->getBoundingBox();
 
  // Attach the entity to the player node
  entityNode = sentSceneMgr->getRootSceneNode()->createChildSceneNode();
  entityNode->attachObject(entity);

  // Scale mesh	and set position and length
  entityNode->setScale(scale,scale,scale);
  entityNode->setPosition(10,-box.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-0); // set position of player node.
  length = box.getSize().x;

  // Perform the ray query
  raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());

  // Set the flags
  if(active)
  {
    entityNode->showBoundingBox(true);
	entity->setQueryFlags(true);
	entity->setVisibilityFlags(true);
  }
  else
  {
    entityNode->showBoundingBox(false);
	entity->setQueryFlags(false);
	entity->setVisibilityFlags(false);
  }
}

void DateArmsArmour::reset()
{ // Resets the armour's flags and position
  entity->setQueryFlags(true);
  entity->setVisible(true);
  entityNode->setPosition(10,-meshBox.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-0);
}

// ****************************************************************************