// Title       : DateBodyArmour.cpp
// Date        : 21/05/13
// Description : The implementation of the class that does the body armour

#include "DateBodyArmour.h"

// ****************************************************************************
// Constructors and Destructors

DateBodyArmour::DateBodyArmour()
{ // Constrcutor

  // Intialises the armour attributes
  name = "";
  scale = 10.0;
  active = true;
  setArmourType(Body);
  objectType = ObjectArmourType;
}

DateBodyArmour::~DateBodyArmour()
{ // Destructor

  if(raySceneQuery)
  { // Destroy the ray query
    sceneMgr->destroyQuery(raySceneQuery);
  }
}

// ****************************************************************************
// Getters and Setters
	
void DateBodyArmour::setPosition(Ogre::Vector3 thePosition)
{ // Ste the position of the armour
  entityNode->setPosition(thePosition);
}

void DateBodyArmour::setActive(bool value)
{ // Set active / inactive
  active = value;
}



// ****************************************************************************
// Create and Reset
void DateBodyArmour::create(Ogre::SceneManager* sentSceneMgr, Ogre::String theName)
{ // Creates the armour

  // Get a copy of the scene manager
  sceneMgr = sentSceneMgr;

  // Set the name of the armour
  name = theName;

  // Create the entity
  entity = sceneMgr->createEntity(name, "Date Model Body 04.mesh");
  Ogre::AxisAlignedBox box = entity->getBoundingBox();
 
  // Attach the entity to the player node
  entityNode = sentSceneMgr->getRootSceneNode()->createChildSceneNode();
  entityNode->attachObject(entity);
		
  //Shirnk mesh
  entityNode->setScale(scale,scale,scale);

  //Set position and length
  entityNode->setPosition(10,-box.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-0); // set position of player node.
  length = box.getSize().x;

  // Perform the ray scene query
  raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());

  // Set the falgs and visibilty depending on active
  if(active)
  {
    entityNode->showBoundingBox(true);
	entity->setQueryFlags(true);
	entity->setVisibilityFlags(true);
  }
  else
  {
    entityNode->showBoundingBox(false);
	entity->setQueryFlags(false);
	entity->setVisibilityFlags(false); 
  }
}

void DateBodyArmour::reset()
{ // Resets the flags and position
  entity->setQueryFlags(true);
  entity->setVisible(true);
  entityNode->setPosition(10,-meshBox.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-0);
}


// ****************************************************************************