/*
-----------------------------------------------------------------------------
Filename:    Game.cpp
-----------------------------------------------------------------------------

This source file is part of the
   ___                 __    __ _ _    _ 
  /___\__ _ _ __ ___  / / /\ \ (_) | _(_)
 //  // _` | '__/ _ \ \ \/  \/ / | |/ / |
/ \_// (_| | | |  __/  \  /\  /| |   <| |
\___/ \__, |_|  \___|   \/  \/ |_|_|\_\_|
      |___/                              
      Tutorial Framework
      http://www.ogre3d.org/tikiwiki/
-----------------------------------------------------------------------------
*/
#include "Game.h"


#include "Titus Sword 1.h"
#include "RecurveBow.h"

#include "TextFileLoader.h"

//-------------------------------------------------------------------------
// Need to pre-declare GUI mouse click subprogram
CEGUI::MouseButton convertMouseButton(OIS::MouseButtonID buttonID);
 
//-------------------------------------------------------------------------
Game::Game(void)
	: mRoot(0), mShutDown(false), mInputManager(0), mMouse(0),
    mKeyboard(0),mCameraMan(0),  
    mSceneMgr(0), mWindow(0),
	mResourcesCfg(Ogre::StringUtil::BLANK),
    mPluginsCfg(Ogre::StringUtil::BLANK),
    globalButtonHeight(0.05),
    smallButtonWidth(0.1),
    largeButtonWidth(0.15),
    sheet(NULL),
    playerHealthText(NULL),
    objText(NULL),
    inventoryText(NULL),
    buttonObjClicked(false),
    buttonInvClicked(false),
    buttonStsClicked(false),
		loadedObjectives(false),
    objectiveName(""),
    objectiveType(""),
    objectiveCompletedString(""),
    objectiveCompleted(false),
    objectiveDescription("")
{
}
//-------------------------------------------------------------------------
Game::~Game(void)
{
	if (mCameraMan) 
		delete mCameraMan;

	if(sword)
		delete sword;

	if(bow)
	  delete bow;

	//if(obeliskHeadArmour)
	//	delete obeliskHeadArmour;
	//if(obeliskArmsArmour)
	//	delete obeliskArmsArmour;
	//if(obeliskBodyArmour)
	//	delete obeliskBodyArmour;
	//if(obeliskLegsArmour)
	//	delete obeliskLegsArmour;
	if(items[0])
		delete items[0];
	if(items[1])
		delete items[1];
	if(items[2])
		delete items[2];
	if(items[3])
		delete items[3];


	//Remove the Window listener
	Ogre::WindowEventUtilities::removeWindowEventListener(mWindow, this);
	windowClosed(mWindow);
	delete mRoot;
}

// should not edit***start ----------------------------------------------------

bool Game::setup()
{
	loadResource(); // load resources - called in setup().

	//ResourceGroupManager::getSingleton().addResourceLocation("C:/Ogre/BSP/BZN/media/sound", "FileSystem",ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, false) ;

	//// configure
	//// Show the configuration dialog and initialise the system
	//if(!(mRoot->restoreConfig() || mRoot->showConfigDialog()))
	//{
	//	return false;
	//}
	//mWindow = mRoot->initialise(true, "BasicTutorial6 Render Window");

	if(mRoot->showConfigDialog())
  {
    // If returned true, user clicked OK so initialise
    // Here we choose to let the system create a default rendering
	  // window by passing 'true'
    mWindow = mRoot->initialise(true, "TutorialApplication Render Window");
  }

	// Set default mipmap level (note: some APIs ignore this)
	Ogre::TextureManager::getSingleton().setDefaultNumMipmaps(5);
	// initialise all resource groups
	Ogre::ResourceGroupManager::getSingleton().initialiseAllResourceGroups();

	// Create the SceneManager, in this case a generic one
  mSceneMgr = mRoot->createSceneManager("DefaultSceneManager");

	// Create the camera
	player.camera = mSceneMgr->createCamera("PlayerCam");
 
	// set the position of the Camera
	player.camera->setPosition(Ogre::Vector3(0,150,200));

	// Look back along -Z
	player.camera->lookAt(Ogre::Vector3(0,0,-300)); 
	player.camera->setNearClipDistance(5);

	//// pitch the camera
	//player.camera->pitch(Ogre::Degree(-25));

	// create a default camera controller
	mCameraMan = new OgreBites::SdkCameraMan(player.camera);   

	// Create one viewport, entire window
	Ogre::Viewport* vp = mWindow->addViewport(player.camera);

	// set colour of the BG
	vp->setBackgroundColour(Ogre::ColourValue(100,0,0));
 
	// Alter the camera aspect ratio to match the viewport
	player.camera->setAspectRatio(
		Ogre::Real(vp->getActualWidth()) / Ogre::Real(vp->getActualHeight()));

 
	// Set ambient light
	mSceneMgr->setAmbientLight(Ogre::ColourValue(0.5, 0.5, 0.5));


	// Create the camera node.
	// starting camera position.
  player.camNode = mSceneMgr->getRootSceneNode()->
		createChildSceneNode("CamNode1"); 
	//player.mCamNode->setPosition(player.cameraVectorFromEntity);
 
  // starting camera direction
  player.camNode->yaw(Ogre::Degree(0));
 
	// attach camera to camera node
  player.camNode->attachObject(player.camera); 

	createScene(); // create the starting environment and objects in the scene.
	               // Called once in setup().

	createFrameListener(); // runs once in the program
	                       // to create a frame listener 
	                       // to this class.


	// setup pointer to sound manager
//  mSoundManager = OgreOggSound::OgreOggSoundManager::getSingletonPtr();
//  mSoundManager->init() ;

//	loadSounds(); // load the sound files needed.

	return true;
}

// load resources - called in setup().
void Game::loadResource()
{
#ifdef _DEBUG
	mResourcesCfg = "resources_d.cfg";
  mPluginsCfg = "plugins_d.cfg";
#else
	mResourcesCfg = "resources.cfg";
    mPluginsCfg = "plugins.cfg";
#endif
 
  // construct Ogre::Root
  mRoot = new Ogre::Root(mPluginsCfg);

	// set up resources
	// Load resource paths from config file
	Ogre::ConfigFile cf;
	cf.load(mResourcesCfg);

	// Go through all sections & settings in the file
	Ogre::ConfigFile::SectionIterator seci = cf.getSectionIterator();
 
	Ogre::String secName, typeName, archName;
	while (seci.hasMoreElements())
	{
		secName = seci.peekNextKey();
		Ogre::ConfigFile::SettingsMultiMap *settings = seci.getNext();
		Ogre::ConfigFile::SettingsMultiMap::iterator i;
		for (i = settings->begin(); i != settings->end(); ++i)
		{
			typeName = i->first;
			archName = i->second;
			Ogre::ResourceGroupManager::getSingleton().addResourceLocation(
				archName, typeName, secName);
		}
	}
}

// load the sound files needed.
void Game::loadSounds()
{
	// the second boolean value enables the looping of the sound.
//	mSoundManager->createSound("music1", "Pokemon - We Will Be Heroes.wav", false, false, false) ;
//  mSoundManager->createSound("music2", "Angel Beats OST - 102 - theme of SSS.wav", false, true, false) ;
//	mSoundManager->createSound("music3", "Naruto OST - The Rising Fighting Spirit.wav", false, true, false) ;
//	mSoundManager->createSound("metalSound", "metalSound.wav", false, false, false) ;
	
}

// initialise OIS for inputmanagment
void Game::initOIS()
{
	Ogre::LogManager::getSingletonPtr()->logMessage("*** Initializing OIS ***");
  OIS::ParamList pl;
  size_t windowHnd = 0;
  std::ostringstream windowHndStr;

  mWindow->getCustomAttribute("WINDOW", &windowHnd);
  windowHndStr << windowHnd;
  pl.insert(std::make_pair(std::string("WINDOW"), windowHndStr.str()));

  mInputManager = OIS::InputManager::createInputSystem( pl );

  mKeyboard = static_cast<OIS::Keyboard*>(mInputManager->
		createInputObject( OIS::OISKeyboard, true ));
  mMouse = static_cast<OIS::Mouse*>(mInputManager->
		createInputObject( OIS::OISMouse, true ));

  mMouse->setEventCallback(this);
  mKeyboard->setEventCallback(this);
}

void Game::createFrameListener(void)
{
	initOIS(); // initialise OIS for inputmanagment


  //Set initial mouse clipping size
  windowResized(mWindow);

  //Register as a Window listener
  Ogre::WindowEventUtilities::addWindowEventListener(mWindow, this);

  mRoot->addFrameListener(this);

	// Populate the camera container
	player.camNode = player.camera->getParentSceneNode();
 

}

//Adjust mouse clipping area
void Game::windowResized(Ogre::RenderWindow* rw)
{
  unsigned int width, height, depth;
  int left, top;
  rw->getMetrics(width, height, depth, left, top);
 
  const OIS::MouseState &ms = mMouse->getMouseState();
  ms.width = width;
  ms.height = height;
}
 
//Unattach OIS before window shutdown (very important under Linux)
void Game::windowClosed(Ogre::RenderWindow* rw)
{
  //Only close for window that created OIS (the main window in these demos)
  if( rw == mWindow )
  {
    if( mInputManager )
    {
      mInputManager->destroyInputObject( mMouse );
      mInputManager->destroyInputObject( mKeyboard );
 
      OIS::InputManager::destroyInputSystem(mInputManager);
      mInputManager = 0;
    }
  }
}

// should not edit***end-------------------------------------------------------

// Create the scene at the beginning of the game
void Game::createScene()
{
	player.create(mSceneMgr, "Player"); // create player entity and associated stuff
	player.setBaseStatsBasedOnLevel(10);
	player.level = 10;
	player.setRealStatsSameAsBaseStats();
	monster.create(mSceneMgr, "monster1");
	monster.entityNode->setPosition(0,20,500);

	//obeliskHeadArmour = new ObeliskHeadArmour;
	//obeliskHeadArmour->create(mSceneMgr, "obeliskHeadArmour");
	//obeliskArmsArmour = new ObeliskArmsArmour;
	//obeliskArmsArmour->create(mSceneMgr, "obeliskArmsArmour");
	//obeliskBodyArmour = new ObeliskBodyArmour;
	//obeliskBodyArmour->create(mSceneMgr, "obeliskBodyArmour");
	//obeliskLegsArmour = new ObeliskLegsArmour;
	//obeliskLegsArmour->create(mSceneMgr, "obeliskLegsArmour");
	items[0] = new ObeliskLegsArmour;
	items[0]->create(mSceneMgr, "Obelisk Helm");
	items[0]->entityNode->setPosition(-300,200,0);
	items[1] = new ObeliskArmsArmour;
	items[1]->create(mSceneMgr, "Obelisk Guards");
	items[1]->entityNode->setPosition(0,175,0);
	items[2] = new ObeliskBodyArmour;
	items[2]->create(mSceneMgr, "Obelisk Plate");
	items[2]->entityNode->setPosition(200,200,0);
	items[3] = new ObeliskHeadArmour;
	items[3]->create(mSceneMgr, "Obelisk Greaves");
	items[3]->entityNode->setPosition(400,200,0);

	sword = new TitusSword1;
	sword->create(mSceneMgr, "Titus Blade");
	bow = new RecurveBow;
	bow->create(mSceneMgr, "Recurvce Bow");

	//map.create(mSceneMgr, "LevelMap"); //create map
	//map.entity->setQueryFlags(false);
	//player.equipArmour((Armour*)items[1]);
	//player.equipArmour((Armour*)items[3]); 

	player.equipWeapon(bow);

		for(int i = 0; i <Gobbies.MAXMONSTER; i++)
	  {
		string num = "gobbie";
		ostringstream convert;   // stream used for the conversion
		convert << i;
		num += convert.str();

		Gobbies.monsters[i].set_Name(num);
		Gobbies.monsters[i].Set_Animation_Names("Attack2","Walk","Idle2");
	  }//end of for loop 

		Gobbies.Group_Location(Ogre::Vector3(180,10,140));
		Gobbies.create(mSceneMgr);

    // **** GUI CREATION

    // Initialise the GUI renderer
    ceguiRenderer = &CEGUI::OgreRenderer::bootstrapSystem();

    // Set the default resource groups
    CEGUI::Imageset::setDefaultResourceGroup("Imagesets");
    CEGUI::Font::setDefaultResourceGroup("Fonts");
    CEGUI::Scheme::setDefaultResourceGroup("Schemes");
    CEGUI::WidgetLookManager::setDefaultResourceGroup("LookNFeel");
    CEGUI::WindowManager::setDefaultResourceGroup("Layouts");

    // Create the scheme
    CEGUI::SchemeManager::getSingleton().create("TaharezLook.scheme");

    // Set the default cursor
    CEGUI::System::getSingleton().setDefaultMouseCursor("TaharezLook", "MouseArrow");

    // Initialise the window manager
    CEGUI::WindowManager &wmgr = CEGUI::WindowManager::getSingleton();

    // Create a sheet
    sheet = wmgr.createWindow("DefaultWindow", "GameGUI/Sheet");

    //Create a quit button
    CEGUI::Window *buttonQuit = wmgr.createWindow("TaharezLook/Button", "OgreP1/QuitButton");
    buttonQuit->setText("Quit");
    buttonQuit->setSize(CEGUI::UVector2(CEGUI::UDim(smallButtonWidth, 0), CEGUI::UDim(globalButtonHeight, 0))); //Top of screen (0, 0)(0, 0)
    buttonQuit->setPosition(CEGUI::UVector2(CEGUI::UDim(0, 0), CEGUI::UDim(0, 0))); //Bottom of screen (?, 0)(0.95, 0)

    // Attach the quit button to the current sheet
    sheet->addChildWindow(buttonQuit);
    CEGUI::System::getSingleton().setGUISheet(sheet);

    buttonQuit->subscribeEvent(CEGUI::PushButton::EventClicked, CEGUI::Event::Subscriber(&Game::quit, this));

    // Create a quest button
    // Displays a text window with the current quest

    buttonObjective = wmgr.createWindow("TaharezLook/Button", "OgreP1/ObjButton");
    buttonObjective->setText("Objectives");
    buttonObjective->setSize(CEGUI::UVector2(CEGUI::UDim(largeButtonWidth, 0), CEGUI::UDim(globalButtonHeight, 0)));
    buttonObjective->setPosition(CEGUI::UVector2(CEGUI::UDim(0,0), CEGUI::UDim(0.95, 0)));
    sheet->addChildWindow(buttonObjective);

    buttonObjective->subscribeEvent(CEGUI::PushButton::EventClicked, CEGUI::Event::Subscriber(&Game::showObjectives, this));

    // Create an inventory button
    // Displays a text window showing players current items

    buttonInventory = wmgr.createWindow("TaharezLook/Button", "OgreP1/InvButton");
    buttonInventory->setText("Inventory");
    buttonInventory->setSize(CEGUI::UVector2(CEGUI::UDim(largeButtonWidth, 0), CEGUI::UDim(globalButtonHeight, 0)));
    buttonInventory->setPosition(CEGUI::UVector2(CEGUI::UDim((largeButtonWidth),0), CEGUI::UDim(0.95, 0)));
    sheet->addChildWindow(buttonInventory);

    buttonInventory->subscribeEvent(CEGUI::PushButton::EventClicked, CEGUI::Event::Subscriber(&Game::showInventory, this));
 

    // Create a stats button
    // Displays a text window listing the players stats

    buttonStats = wmgr.createWindow("TaharezLook/Button", "OgreP1/StatsButton");
    buttonStats->setText("Character");
    buttonStats->setSize(CEGUI::UVector2(CEGUI::UDim(largeButtonWidth, 0), CEGUI::UDim(globalButtonHeight, 0)));
    buttonStats->setPosition(CEGUI::UVector2(CEGUI::UDim((largeButtonWidth*2),0), CEGUI::UDim(0.95, 0)));
    sheet->addChildWindow(buttonStats);

    buttonStats->subscribeEvent(CEGUI::PushButton::EventClicked, CEGUI::Event::Subscriber(&Game::showStats, this));

    // Create text to display health
    try
    {
      playerHealthText = CEGUI::WindowManager::getSingleton().createWindow("TaharezLook/StaticText", "OgreP1/HealthText");
      string healthString = "HP  ";
      healthString += convertIntToString(player.realHealthStt);
      playerHealthText->setText(healthString);
      playerHealthText->setSize(CEGUI::UVector2(CEGUI::UDim(0.15, 0), CEGUI::UDim(0.05, 0)));
      playerHealthText->setPosition(CEGUI::UVector2(CEGUI::UDim(0.85, 0), CEGUI::UDim(0, 0)));
      sheet->addChildWindow(playerHealthText);
    }
    catch (exception &e)
    {
      string message = "\nException in second playerHealthText";
      Ogre::LogManager::getSingleton().logMessage(message);
    }

    // **** END OF GUI CREATION

	Ogre::Plane plane(Ogre::Vector3::UNIT_Y, 0); // create a 2D plane.

	// create ground mesh
	Ogre::MeshManager::getSingleton().createPlane("ground", 
		Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
		plane, 1500, 1500, 20, 20, true, 1, 5, 5, Ogre::Vector3::UNIT_Z);

	// create a ground entity using the ground mesh
	Ogre::Entity* entGround = mSceneMgr->createEntity("GroundEntity", "ground");

	// attach ground entity to a node.
	Ogre::SceneNode* sceneNodeGround = mSceneMgr->getRootSceneNode()->createChildSceneNode();
	sceneNodeGround->attachObject(entGround);

	// set ground texture
	entGround->setMaterialName("Examples/Rockwall");
	entGround->setCastShadows(false); // disable casting shadow

	sceneNodeGround->showBoundingBox(true);

    bMap.createMainFloorPlane(mSceneMgr);
	bMap.createTunnelPlane(mSceneMgr, "tunnelWall",  Ogre::Radian(Ogre::Real(Ogre::Math::PI/2-Ogre::Math::PI)), 0, 250, 0);
	bMap.createTunnelPlane(mSceneMgr, "tunnelWall2", Ogre::Radian(Ogre::Real(Ogre::Math::PI)), 250, 500, 0);
	bMap.createTunnelPlane(mSceneMgr, "tunnelWall3", Ogre::Radian(Ogre::Real(Ogre::Math::PI/2)), 500, 250, 0);
	bMap.createCavernDoorPlaneMiddle(mSceneMgr, "cavernWallMiddle1", Ogre::Vector3::UNIT_X, Ogre::Radian(Ogre::Real(Ogre::Math::PI/2-Ogre::Math::PI)), 250, 1750, -1500);
	bMap.createCavernDoorPlaneLeft(mSceneMgr, "cavernWallLeft1", Ogre::Vector3::UNIT_X, Ogre::Radian(Ogre::Real(Ogre::Math::PI/2-Ogre::Math::PI)), -750, 1500, -1500);
	bMap.createCavernDoorPlaneRight(mSceneMgr, "cavernWallRight1", Ogre::Vector3::UNIT_X, Ogre::Radian(Ogre::Real(Ogre::Math::PI/2-Ogre::Math::PI)), 1000, 1500, -1500);
	
	bMap.createCavernPlane(mSceneMgr, "cavernWall1", Ogre::Vector3::UNIT_Z, Ogre::Radian(Ogre::Real(Ogre::Math::PI/2-Ogre::Math::PI)), -1500, 1500, -3000);
	bMap.createCavernPlane(mSceneMgr, "cavernWall2", Ogre::Vector3::UNIT_X, Ogre::Radian(Ogre::Real(Ogre::Math::PI/2)), 0, 1500, -4500);
	bMap.createCavernPlane(mSceneMgr, "cavernWall3", Ogre::Vector3::UNIT_Z, Ogre::Radian(Ogre::Real(Ogre::Math::PI/2)), 1500, 1500, -3000);
}

//-----------------------------------------------------------------------------
// subprogram used to run the game
void Game::go(void)
{
    if (!setup()) //setup the game
        return;

    mRoot->startRendering(); // start the game loop
}

//-----------------------------------------------------------------------------

// Subprogram run every frame to update.
// All object requires update should be done here, 
// to reduce objects inheriting Ogre::FrameListener class.
bool Game::frameRenderingQueued(const Ogre::FrameEvent& evt)
{
  if(mWindow->isClosed())
    return false;

  if(mShutDown) 
    return false;
 
  //Need to capture/update each device
  mKeyboard->capture();
  mMouse->capture();
 
  player.update(evt); // update player stuff
  monster.update(); 

  for(int i = 0; i<4; i++)
  {
	  for(int j = 0; j<8;j++)
	  {
		  if(Gobbies.monsters[i].isCollision(Gobbies.monsters[i].entityNode->getPosition(),
			  Ogre::Vector3::NEGATIVE_UNIT_Y,player.arrowCollection.arrows[j].name))
		  {
			  Gobbies.monsters[i].entity->setVisible(false);
			  Ogre::LogManager::getSingleton().logMessage("\n arrow collide with ninja \n");
		  }
		  //player.arrowCollection.arrows[j].is
	  }
	  
  }

  for(int j = 0; j<8;j++)
  {
		  if(monster.isCollision(monster.entityNode->getPosition(), Ogre::Vector3::NEGATIVE_UNIT_Y, player.arrowCollection.arrows[j].name))
		  {
			  monster.entity->setVisible(false);
			  Ogre::LogManager::getSingleton().logMessage("\n arrow collide with monster \n");
		  }
		  //player.arrowCollection.arrows[j].is
  }

	player.pickUp(items,4);

	if(!player.isCollision(player.entityNode->getPosition(), items,4,
		player.entityNode->getOrientation() * Ogre::Vector3(0, 0, 1))
		&&
		!player.isCollision(player.entityNode->getPosition(),
		player.entityNode->getOrientation() * Ogre::Vector3(0, 0, 1),"monster1"))
	{
		player.prePosition = player.playerPosition;
		player.preCamPosition = player.camNode->getPosition();		
		for(int i =0; i<4;i++)
		{
			items[i]->addGravity(evt);
		}
		player.addGravity(evt);
		monster.addGravity(evt);
		player.translate(evt);


	}
	else
	{
		//reset positions to positions before collision.

		player.bodyArmour->entityNode->setPosition(player.prePosition);
		player.headArmour->entityNode->setPosition(player.prePosition);
		player.legsArmour->entityNode->setPosition(player.prePosition);
		player.armsArmour->entityNode->setPosition(player.prePosition);
		player.entityNode->setPosition(player.prePosition);
		player.camNode->setPosition(player.preCamPosition);

		        // Test take damage on a collision
        player.takeDamage(3, 7);
	}

  player.updateRightHand(evt);

	for(int i = 0; i <Gobbies.MAXMONSTER; i++)
  {
	Gobbies.monsters[i].Find_Player_Location(player.getPosition());
  }//end of for loop
  Gobbies.update(evt);

	updateHealthDisplay();

  if (this->buttonObjClicked)
  {
    updateObjectiveDisplay();
  }

  if (this->buttonInvClicked)
  {
    updateInventoryDisplay();
  }

  if (this->buttonStsClicked)
  {
    updateStatDisplay();
  }

  static bool doOnce = false;

  if (doOnce == false)
  {
    // mSoundManager->getSound("music1")->play();
    doOnce = true;
  }//end of if

  // Update the frame time in the GUI system
  CEGUI::System::getSingleton().injectTimePulse(evt.timeSinceLastFrame);

  return true;
}//end of frameRenderingQueued

//-----------------------------------------------------------------------------
// OIS::KeyListener
bool Game::keyPressed( const OIS::KeyEvent& evt )
{static int doOnce = 0;
  switch (evt.key)
  {
		case OIS::KC_ESCAPE: 
			//mShutDown = true;
            if (buttonObjClicked)
            {
              objText->destroy();
              buttonObjClicked= false;
              buttonInventory->enable();
              buttonStats->enable();
            }
            if (buttonInvClicked)
            {
              inventoryText->destroy();
              buttonInvClicked = false;
              buttonObjective->enable();
              buttonStats->enable();
            }
            if (buttonStsClicked)
            {
              statsText->destroy();
              buttonStsClicked = false;
              buttonObjective->enable();
              buttonInventory->enable();
            }
			break;
		case OIS::KC_UP:
		case OIS::KC_W:
			player.accelerateValue.z = -player.moveValue;
			break;
 
		case OIS::KC_DOWN:
		case OIS::KC_S:
			player.accelerateValue.z = player.moveValue;
			break;
 
		case OIS::KC_LEFT:
		case OIS::KC_A:
			player.accelerateValue.x = -player.moveValue;
			break;
 
		case OIS::KC_RIGHT:
		case OIS::KC_D:
			player.accelerateValue.x = player.moveValue;
			break;
 
		case OIS::KC_PGDOWN:
		case OIS::KC_E:
			player.accelerateValue.y = -player.moveValue;
			break;
 
		case OIS::KC_PGUP:
		case OIS::KC_Q:
			player.accelerateValue.y = player.moveValue;
			break;

		case OIS::KC_C:
			player.rightHandPitchValue -= 150;
			break;
		case OIS::KC_X:
			player.rightHandPitchValue += 150;
			break;
		case OIS::KC_V:
			if(player.isAttack==false)
			{
				player.isAttack = true;
			}
			break;
		case OIS::KC_F:
			if(player.isAttacking==false)
			{
				player.isAttack = true;
			}
			break;
			
	  case OIS::KC_B:
			monster.reset();
			player.reset();
			bow->reset();
			sword->reset();

			break;
		case OIS::KC_N:

			if(doOnce==0)
			{doOnce=1;
			player.equipWeapon(sword);
			}
			else if(doOnce==1)
			{doOnce=0;
			player.equipWeapon(bow);
			}
			break;
		case OIS::KC_R:
			/*map.entity->setQueryFlags(false);*/
			break;
		case OIS::KC_T:
			/*map.entity->setQueryFlags(true);*/
			break;

		case OIS::KC_Z:
			if(player.isJumping==false)
				player.isJumping=true;
			break;
		case OIS::KC_0:
			player.unequipArmour(0);
			break;
		case OIS::KC_1:
			player.unequipArmour(1);
			break;
		case OIS::KC_2:
			player.unequipArmour(2);
			break;
		case OIS::KC_3:
			player.unequipArmour(3);
			break;
		case OIS::KC_4:
			player.unequipWeapon();
			break;

		default:
			break;
  }
  return true;
}
bool Game::keyReleased( const OIS::KeyEvent& evt )
{
	switch(evt.key)
	{
		case OIS::KC_UP:
		case OIS::KC_W:
			player.accelerateValue.z = 0;
			break;
 
		case OIS::KC_DOWN:
		case OIS::KC_S:
			player.accelerateValue.z = 0;
			break;
 
		case OIS::KC_LEFT:
		case OIS::KC_A:
			player.accelerateValue.x = 0;
			break;
 
		case OIS::KC_RIGHT:
		case OIS::KC_D:
			player.accelerateValue.x = 0;
			break;
 
		case OIS::KC_PGDOWN:
		case OIS::KC_E:
			player.accelerateValue.y = 0;
			break;
 
		case OIS::KC_PGUP:
		case OIS::KC_Q:
			player.accelerateValue.y = 0;
			break;
 
		case OIS::KC_C:
			player.rightHandPitchValue = 0;
			break;
		case OIS::KC_X:
			player.rightHandPitchValue = 0;
			break;

		default:
			break;
	}

	return true;

}

//-----------------------------------------------------------------------------
// OIS::MouseListener
bool Game::mouseMoved( const OIS::MouseEvent& evt )
{
    CEGUI::System &sys = CEGUI::System::getSingleton();
    sys.injectMouseMove(evt.state.X.rel, evt.state.Y.rel);

	if (evt.state.buttonDown(OIS::MB_Right))
	{
		player.rotate(evt);
	}

    if (evt.state.Z.rel)
    {
      sys.injectMouseWheelChange(evt.state.Z.rel / 120.0f);
    }
	return true;
}
bool Game::mousePressed( const OIS::MouseEvent& evt, OIS::MouseButtonID id )
{
    CEGUI::System::getSingleton().injectMouseButtonDown(convertMouseButton(id));
	return true;
}
bool Game::mouseReleased( const OIS::MouseEvent& evt, OIS::MouseButtonID id )
{
    CEGUI::System::getSingleton().injectMouseButtonUp(convertMouseButton(id));
	return true;
}

CEGUI::MouseButton convertMouseButton(OIS::MouseButtonID buttonID)
{
  if (buttonID == OIS::MB_Left)
  {
    return CEGUI::LeftButton;
  }
  else if (buttonID == OIS::MB_Right)
  {
    return CEGUI::RightButton;
  }
  else if (buttonID == OIS::MB_Middle)
  {
    return CEGUI::MiddleButton;
  }
  else
  {
    return CEGUI::LeftButton;
  }
}

bool Game::quit(const CEGUI::EventArgs &e)
{
  mShutDown = true;
  return true;
}

bool Game::showObjectives(const CEGUI::EventArgs &e)
{
  try
  {
    buttonObjClicked = true;
    buttonInventory->disable();
    buttonStats->disable();
    objText = CEGUI::WindowManager::getSingleton().createWindow("TaharezLook/StaticText", "OgreP1/objText");
    updateObjectiveDisplay();
    objText->setSize(CEGUI::UVector2(CEGUI::UDim(0.5, 0), CEGUI::UDim(0.5, 0)));
    objText->setPosition(CEGUI::UVector2(CEGUI::UDim(0.475, 0), CEGUI::UDim(0.475, 0)));
    sheet->addChildWindow(objText);
  }
  catch (exception &ex)
  {
    string message = "\nException in showObjectives";
    Ogre::LogManager::getSingleton().logMessage(message);
    return false;
  }

  return true;
}

bool Game::showInventory(const CEGUI::EventArgs &e)
{
   try
   {
    buttonObjective->disable();
    buttonStats->disable();
    inventoryText = CEGUI::WindowManager::getSingleton().createWindow("TaharezLook/StaticText", "OgreP1/invText");
    updateInventoryDisplay();
    inventoryText->setSize(CEGUI::UVector2(CEGUI::UDim(0.5, 0), CEGUI::UDim(0.5, 0)));
    inventoryText->setPosition(CEGUI::UVector2(CEGUI::UDim(0.475, 0), CEGUI::UDim(0.475, 0)));
    sheet->addChildWindow(inventoryText);
    buttonInvClicked = true;
  }
  catch (exception &ex)
  {
    string message = "\nException in showInv";
    Ogre::LogManager::getSingleton().logMessage(message);
    return false;
  }

  return true; 
}

bool Game::showStats(const CEGUI::EventArgs &e)
{
  try
   {
    buttonObjective->disable();
    buttonInventory->disable();
    statsText = CEGUI::WindowManager::getSingleton().createWindow("TaharezLook/StaticText", "OgreP1/statsText");
    updateStatDisplay();
    statsText->setSize(CEGUI::UVector2(CEGUI::UDim(0.5, 0), CEGUI::UDim(0.5, 0)));
    statsText->setPosition(CEGUI::UVector2(CEGUI::UDim(0.475, 0), CEGUI::UDim(0.475, 0)));
    sheet->addChildWindow(statsText);
    buttonStsClicked = true;
  }
  catch (exception &ex)
  {
    string message = "\nException in showSts";
    Ogre::LogManager::getSingleton().logMessage(message);
    return false;
  }

  return true; 
}

string Game::convertIntToString(int number)
{
  std::ostringstream buffer;
  buffer << number;
  return buffer.str();
}
void Game::updateHealthDisplay()
{
  string playerHealthString = "HP: ";
  playerHealthString += convertIntToString(player.realHealthStt);
  playerHealthText->setText(playerHealthString);
}


void Game::updateObjectiveDisplay()
{
  if (!loadedObjectives)
  {
    TextFileLoader objectiveLoader;
    objectiveLoader.ObjectiveLoader(objectiveName, objectiveType, objectiveCompleted, objectiveDescription);
    loadedObjectives = true;
  }

  if (objectiveCompleted == false)
  {
    objectiveCompletedString = "Not completed";
  }
  else
  {
    objectiveCompletedString = "Completed";
  }

  string objString = "Current Objective\n";
  objString += "    \n";
  objString += objectiveName;
  objString += "\n";
  objString += "    \n";
  objString += "\nObjective Type\n ";
  objString += objectiveType;
  objString += "\n";
  objString += "    \n";
  objString += "\nObjective completed\n ";
  objString += objectiveCompletedString;
  objString += "\n";
  objString += "    \n";
  objString += "\nHint\n ";
  objString += objectiveDescription;
  objText->setText(objString);  
}


void Game::updateInventoryDisplay()
{
    string invString = "Current Inventory\n";
    invString += "    \n";
    invString += "\nWeapon\n ";
    invString += player.equippedWeapon->getName();
    invString += "\n";
    invString += "    \n";
    invString += "\nHead Armour\n ";
    invString += player.headArmour->getName();
    invString += "\n";
    invString += "    \n";
    invString += "\nArms Armour\n ";
    invString += player.armsArmour->getName();
    invString += "\n";
    invString += "    \n";
    invString += "\nBody Armour\n ";
    invString += player.bodyArmour->getName();
    invString += "\n";
    invString += "    \n";
    invString += "\nLeg Armour\n ";
    invString += player.legsArmour->getName();
    inventoryText->setText(invString);
}

void Game::updateStatDisplay()
{
    string statsString = "Current Stats\n ";
    statsString += "    \n";
    statsString += "HP\n ";
    statsString += convertIntToString(player.realHealthStt);
    statsString += "/";
    statsString += convertIntToString(player.baseHealthStt);
    statsString += "\n";
    statsString += "    \n";
    statsString += "\nAttack\n ";
    statsString += convertIntToString(player.realAttackStt);
    statsString += "\n";
    statsString += "    \n";
    statsString += "\nDefence\n ";
    statsString += convertIntToString(player.realDefenceStt);
    statsString += "\n";
    statsString += "    \n";
    statsString += "\nDexterity\n ";
    statsString += convertIntToString(player.realDextStt);
    statsString += "\n";
    statsString += "    \n";
    statsString += "\nSpeed\n ";
    statsString += convertIntToString(player.realSpeedStt);
    statsText->setText(statsString);
}

//-------------------------------------------------------------------------

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#define WIN32_LEAN_AND_MEAN
#include "windows.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
  INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR strCmdLine, INT )
#else
  int main(int argc, char *argv[])
#endif
  {
    // Create application object
    //TutorialApplication app;
		Game app;

    try 
		{
      app.go();
        
		} 
		catch( Ogre::Exception& e ) 
		{
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
            MessageBox( NULL, e.getFullDescription().c_str(), 
				"An exception has occured!", MB_OK | MB_ICONERROR | MB_TASKMODAL);
#else
            std::cerr << "An exception has occured: " <<
                e.getFullDescription().c_str() << std::endl;
#endif
    }

    return 0;
  }

#ifdef __cplusplus
}
#endif
