
#ifndef __Game_h_
#define __Game_h_

#pragma warning( disable: 4251 )

#include "Player.h"
#include "Monster.h"

#include "Map.h"
#include "ObeliskArmsArmour.h"
#include "ObeliskBodyArmour.h"
#include "ObeliskHeadArmour.h"
#include "ObeliskLegsArmour.h"

#include "AIMonster.h"

#include "BuildMap.h"

//#include "OgreOggSound.h"

//using namespace OgreOggSound;

// Includes for the GUI
#include <CEGUI.h>
#include <RendererModules\Ogre\CEGUIOgreRenderer.h>

// The Main Class of the game. All game instances are in this class.
class Game: public Ogre::WindowEventListener, 
						public Ogre::FrameListener, 
						public OIS::KeyListener, 
						public OIS::MouseListener
						//, OgreBites::SdkTrayListener
{
private:
	// Ogre classes used to run the program
	Ogre::Root* mRoot;
  Ogre::String mPluginsCfg;
	Ogre::String mResourcesCfg;
	Ogre::RenderWindow* mWindow;
	Ogre::SceneManager* mSceneMgr;

	// Used to play sounds
	//OgreOggSound::OgreOggSoundManager* mSoundManager ; 


	// OIS Input devices
	OIS::InputManager* mInputManager;
	OIS::Mouse*    mMouse;
	OIS::Keyboard* mKeyboard;


	bool mShutDown; // used to tell the game to shutdown.

	Player player; // Player instance - player character + player camera
	AIMonster_Collection Gobbies; //collection of goblins
	Monster monster;

    BuildMap bMap;

	Object* items[4];
	//Object* obeliskHeadArmour;
	//Object* obeliskArmsArmour;
	//Object* obeliskBodyArmour;
	//Object* obeliskLegsArmour;
	Weapon* sword;
	Weapon* bow;

	// create the starting environment and objects in the scene.
	// Called once in setup().
	void createScene(); 

	
	// Ogre::FrameListener
	virtual bool frameRenderingQueued(const Ogre::FrameEvent& evt);
	// Subprogram run every frame to update.
	// All object requires update should be done here, 
	// to reduce objects inheriting Ogre::FrameListener class.

	// OgreBites
	OgreBites::SdkCameraMan* mCameraMan;       // basic camera controller


  // OIS::KeyListener
  virtual bool keyPressed( const OIS::KeyEvent& evt );
  virtual bool keyReleased( const OIS::KeyEvent& evt );
  // OIS::MouseListener
  virtual bool mouseMoved( const OIS::MouseEvent& evt );
  virtual bool mousePressed( const OIS::MouseEvent& evt, OIS::MouseButtonID id );
  virtual bool mouseReleased( const OIS::MouseEvent& evt, OIS::MouseButtonID id );


// should not edit***start

	// Ogre::WindowEventListener
	virtual void windowResized(Ogre::RenderWindow* rw);
	virtual void windowClosed(Ogre::RenderWindow* rw);

	// setup the ogre engine
	bool setup();
	void loadResource(); // load resources - called in setup().
	void loadSounds();

	// Ogre::FrameListener
	virtual void createFrameListener(void); // runs once in the program
	                                        // to create a frame listener 
	                                        // to this class.

	void initOIS(); // setup OIS input management used for 
	                // keybroad + mouse detection.
	                // Called in createFrameListener.
// should not edit***end

// **** All Public GUI Stuff ****

   // GUI Sheet
    CEGUI::Window *sheet;

    // Button Sizes
    const float globalButtonHeight;
    const float smallButtonWidth;
    const float largeButtonWidth;

    // Button objects
    CEGUI::Window *buttonObjective;
    CEGUI::Window *buttonInventory;
    CEGUI::Window *buttonStats;

    //Button Events
    bool quit(const CEGUI::EventArgs &e);
    bool showObjectives(const CEGUI::EventArgs &e);
    bool showInventory(const CEGUI::EventArgs &e);
    bool showStats(const CEGUI::EventArgs &e);

    // Button Event Text
    CEGUI::Window* playerHealthText;
    CEGUI::Window* objText;
    CEGUI::Window* inventoryText;
    CEGUI::Window* statsText;

    // Button Active
    bool buttonObjClicked;
    bool buttonInvClicked;
    bool buttonStsClicked;

		// Update text
    void updateHealthDisplay();
    void updateObjectiveDisplay();
    void updateInventoryDisplay();
    void updateStatDisplay();

		// Load/re-loaded objective
    bool loadedObjectives;

		// Objective stuff
    string objectiveName;
    string objectiveType;
    string objectiveCompletedString;
    bool objectiveCompleted;
    string objectiveDescription;
    
    // Utils
    string convertIntToString(int number);

// **** End of All Public GUI Stuff ****

protected:

// **** All Protected GUI Stuff ****

  // Pointer to the GUI renderer
  CEGUI::OgreRenderer* ceguiRenderer;

// **** End of All Protected GUI Stuff ****
public:
	//constructor
  Game(void);
  virtual ~Game(void);

	// subprogram used to run the game
  void go(void);
};

#endif // #ifndef __Game_h_
