// Title       : ObeliskBodyArmour.cpp
// Date        : 21/05/13
// Description : The class that does the Obelisk body armour

#include "ObeliskBodyArmour.h"

// ****************************************************************************
// Constructors and Destructors

ObeliskBodyArmour::ObeliskBodyArmour()
{ // Constructor

  // Set the armour attributes
  name = "";
  scale = 10.0;
  active = true;
  setArmourType(Body);
  this->setDenfenceStt(100);
  displacementFromGround = -40;
  objectType = ObjectArmourType;
}

ObeliskBodyArmour::~ObeliskBodyArmour()
{ // Destructor

  if(raySceneQuery)
  { // Destroy the ray query
    sceneMgr->destroyQuery(raySceneQuery);
  }
}

// ****************************************************************************
// Getters and Setters

void ObeliskBodyArmour::setPosition(Ogre::Vector3 thePosition)
{ // Sets the position of the armour
  entityNode->setPosition(thePosition);
}

void ObeliskBodyArmour::setActive(bool value)
{ // Sets armour active / inactive
  active = value;
}

// ****************************************************************************
// Create and Reset

void ObeliskBodyArmour::create(Ogre::SceneManager* sentSceneMgr, Ogre::String theName)
{ // Creates the armour

  // Get a copy of the scene manager
  sceneMgr = sentSceneMgr;

  // Set the name of the armour
  name = theName;

  // Create the entity
  entity = sceneMgr->createEntity(name, "Obelisk Body 03.mesh");

  // Attach the entity to play node		
  entityNode = sentSceneMgr->getRootSceneNode()->createChildSceneNode();
  entityNode->attachObject(entity);

  // Bounding box
  Ogre::AxisAlignedBox box = entity->getBoundingBox();

  // Scale the mesh
  entityNode->setScale(scale,scale,scale);

  // Set position
  entityNode->setPosition(10,-box.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-0);

  // Set length
  length = box.getSize().x;

  // Perform the ray query
  raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());

  // Set the flags and visibility depending on if active
  if(active)
  {
    entityNode->showBoundingBox(true);
	entity->setQueryFlags(true); // turn on/off query for this entity used for collision detection.
	entity->setVisibilityFlags(true); // turn visiblility on/off for this entity.
  }
  else
  {
    entityNode->showBoundingBox(false);
	entity->setQueryFlags(false); // turn on/off query for this entity used for collision detection.
    entity->setVisibilityFlags(false); // turn visiblility on/off for this entity.
  }
}

void ObeliskBodyArmour::reset()
{ // Reset the flags and position
  entity->setQueryFlags(true);
  entity->setVisible(true);
  entityNode->setPosition(10,-meshBox.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-0); // set position of player node.
}
// ****************************************************************************