
#include <OgreRoot.h>
#include <OISEvents.h>
#include <OISInputManager.h>
#include <OISKeyboard.h>
#include <OISMouse.h>
#include <OgreWindowEventUtilities.h>


#include <OgreConfigFile.h>
#include <OgreException.h>

#include <OgreCamera.h>
#include <OgreViewport.h>
#include <OgreSceneManager.h>
#include <OgreRenderWindow.h>
#include <OgreEntity.h>
#include <OgreResourceGroupManager.h>


//#include <SdkTrays.h>
#include <SdkCameraMan.h>

#include <string>
using namespace std;
#pragma once

const int MONSTERLISTSIZE = 1;
const Ogre::String MONSTERLIST[MONSTERLISTSIZE] = {"monster1"};

const int ITEMLISTSIZE = 6;
const Ogre::String ITEMLIST[ITEMLISTSIZE] = {"obeliskHeadArmour", "obeliskArmsArmour",
                                             "obeliskBodyArmour", "obeliskLegsArmour",
                                             "titusSword", "recurvceBow"};


enum ObjectType {ObjectWeaponType, ObjectArmourType, ObjectMonsterType, ObjectNone};

//This class is an abstract version of all objects in the game, this includes
//player, mobs, sound, camera etc
class Object
{
private:
	string collidedObject;
public:	

	ObjectType objectType; // Used to tell what kind of object it is. E.g. Weapon.

	string name;	//Name of Object
	//Ogre::Vector3 direction;  // Value to move in the correct direction
	//													// A.K.A. Position

	Ogre::Entity* entity;         // The Entity we are animating
  Ogre::SceneNode* entityNode;  // The SceneNode that the Entity is attached to
	Ogre::SceneManager* sceneMgr;

	Ogre::RaySceneQuery *raySceneQuery; // Used for collision detection
	Ogre::Real length;
	Ogre::Real scale;
	Ogre::Real displacementFromGround;

	Ogre::Vector3 prePosition; // Used revert back a postion before collision.

	bool active;

	Ogre::AxisAlignedBox meshBox;

	
	string getCollidedObject();

	void removeColliedOBject();

	Object();

	virtual void create(Ogre::SceneManager* sentSceneMgr, Ogre::String theName) = 0;

	
	// collision detection
	bool isCollision(const Ogre::Vector3& position, const Ogre::Vector3& direction);

	
	// collision detection
	bool isCollision(const Ogre::Vector3& position, const Ogre::Vector3& direction, 
		Ogre::String nameOfObject);

	// collision detection
	bool isCollisionMonster(const Ogre::Vector3& position, const Ogre::Vector3& direction);

	virtual void reset() = 0;

	bool isCollision(const Ogre::Vector3& position, Object* objects[], int numberOfObjects,
		const Ogre::Vector3& direction = Ogre::Vector3::NEGATIVE_UNIT_Y) ;

	virtual void addGravity(const Ogre::FrameEvent& evt)
	{
		if(entityNode)
		{
			
			prePosition = entityNode->getPosition();
			if(!isCollision(entityNode->getPosition()-Ogre::Vector3(0,displacementFromGround,0),
				entityNode->getOrientation() * Ogre::Vector3(0, -2, 0), "GroundEntity"))
			{
				
				entityNode->translate(0,-100*evt.timeSinceLastEvent,0);
			}
			else
			{
				//reset positions to positions before collision.

				entityNode->setPosition(prePosition);
			}
		}
	}
};



