#include "Creature.h"
#include "Weapon.h"
#include "ArrowCollection.h"
#include "Armour.h"
#include "DateArmsArmour.h"
#include "DateBodyArmour.h"
#include "DateHeadArmour.h"
#include "DateLegsArmour.h"


#pragma once

class SkillTree;

class Player: public Creature
{
private:
	int currentWeightStt;
	int maximumWeightStt;
	int numShineyRock;

	SkillTree* skillTree;
	
public:
	Ogre::Camera* camera;     // player's camera.
	Ogre::SceneNode *camNode; // The SceneNode the camera is 
	                           // currently attached to.

	Ogre::Vector3 accelerateValue;
	Ogre::Vector3 cameraVectorFromEntity;

	
	Ogre::Vector3 armsVectorFrombody;
	Ogre::Vector3 startingPosition;
	Ogre::Vector3 rightHandVectorFromArm;
	Ogre::Vector3 preCamPosition;

	// 
	Armour* defaultArmsArmour;
	Armour* defaultBodyArmour;
	Armour* defaultHeadArmour;
	Armour* defaultLegsArmour;

	Armour* headArmour;
	Armour* bodyArmour;
	Armour* armsArmour;
	Armour* legsArmour;
	Ogre::Vector3 playerPosition;
	Ogre::Vector3 armsPosition;

	Ogre::Quaternion playerOrientation;
	Ogre::Quaternion armsOrientation;

	bool weaponEquipped; // return a boolean stating if a weapon is equipped.
	bool headEquipped;   // return a boolean stating if a head armour is equipped.
	bool bodyEquipped;   // return a boolean stating if a body armour is equipped.
	bool armsEquipped;   // return a boolean stating if a arms armour is equipped.
	bool legsEquipped;   // return a boolean stating if a legs armour is equipped.

	Weapon* equippedWeapon;// current equipped weapon.

	Ogre::Real rightHandPitchValue;
	Ogre::Real rightHandPitchTotal;

	bool isAttack;
	bool isAttacking;
	double meleeAttackTime;
	double jumpTime;
	bool isJumping;

	ArrowCollection arrowCollection;



	void equipWeapon(Weapon* weapon);

	void equipArmour(Armour* armour);

	void updateRightHand(const Ogre::FrameEvent& evt);

	Player();

	Player(int requestedLevel);

	~Player();

	void pickUp();


	void create(Ogre::SceneManager* sentSceneMgr, Ogre::String theName);

	void update(const Ogre::FrameEvent& evt);


	void translate(const Ogre::FrameEvent& evt);

	void takeDamage(int enemyAttackDamage, int enemySpeed);

//*****************************************************************************
//player Controls
//*****************************************************************************


	void rotate( const OIS::MouseEvent& evt);

	void updateRealStats();

	void reset();

	void addGravity(const Ogre::FrameEvent& evt);

	void pickUp(Object* items[], int numberOfItems);

	void unequipWeapon();

	void unequipArmour(int parts); // 0 = head, 1 = body, 2 = arms, 3 = legs
};