// Title       : RecurveBow.cpp
// Date        : 21/05/13
// Description : The implementation of the RecurveBow class

#include "RecurveBow.h"

// ****************************************************************************
// Constructors and Destructors

RecurveBow::RecurveBow()
{ // Constructor

  name = "";
  scale = 10.0;
  this->setAccu(20);
  this->setAtt(20);
  objectType = ObjectWeaponType;
}

RecurveBow::~RecurveBow()
{ // Destructor

  if(raySceneQuery)
  { // Destroy the ray scene query
    sceneMgr->destroyQuery(raySceneQuery);
  }
}

// ****************************************************************************
// Getters and Setters

void RecurveBow::setPosition(Ogre::Vector3 thePosition)
{ // Set the position of the weapon
  entityNode->setPosition(thePosition);
}

void RecurveBow::setActive(bool value)
{ // Sets the weapon to active
  active = value;
}

// ****************************************************************************
// Create and Reset

void RecurveBow::create(Ogre::SceneManager* sentSceneMgr, Ogre::String theName)
{ // Creates the bow

  sceneMgr = sentSceneMgr; // Get a copy of sceneManager

  name = theName;

  this->setWeaponType(Bow); // Set the weapon type

  // Create the entity
  entity = sceneMgr->createEntity(name, "Bow_Recurve_Rigged_01.mesh");
  meshBox = entity->getBoundingBox();
 
  entityNode = sentSceneMgr->getRootSceneNode()->createChildSceneNode();
  entityNode->attachObject(entity); // Attach player's entity to player node
		
		
  entityNode->setScale(scale,scale,scale); // Make player's mesh smaller.

  // Set the position of the player node
  entityNode->setPosition(-100,-meshBox.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-400);
  length = meshBox.getSize().x;

  raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());
		
  if(active)
  {
    entity->setQueryFlags(true); // turn on/off query for this entity used for collision detection.
    entity->setVisibilityFlags(true); // turn visiblility on/off for this entity.
  }
  else
  {
    entityNode->showBoundingBox(false);
	entity->setQueryFlags(false); // turn on/off query for this entity used for collision detection.
	entity->setVisibilityFlags(false); // turn visiblility on/off for this entity.
  }
}

void RecurveBow::reset()
{ // Resets the bow
  
  // Set flags, visible and active
  entity->setQueryFlags(true);
  entity->setVisible(true);
  active = true;

  // Reset the position of the player node
  entityNode->setPosition(-100,-meshBox.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-400);
}

// ****************************************************************************




