// Title       : TitusSword1.cpp
// Date        : 21/05/13
// Description : The implementation of the TitusSword class

#include "Titus Sword 1.h"

// ****************************************************************************
// Constructors and Destructors

TitusSword1::TitusSword1()
{ // Constructor

  // Set the attributes of the sword to defaults
  name = "";
  scale = 10.0;
  this->setAccu(10);
  this->setAtt(40);
  objectType = ObjectWeaponType;
}

TitusSword1::~TitusSword1()
{ // Destructor

  if(raySceneQuery)
  { // Destroy the ray scene query
    sceneMgr->destroyQuery(raySceneQuery);
  }
}

// ****************************************************************************
// Getters and Setters

void TitusSword1::setPosition(Ogre::Vector3 thePosition)
{ // Sets the position of the weapon

  entityNode->setPosition(thePosition);
}

void TitusSword1::setActive(bool value)
{ // Sets if the weapon is active
  active = value;
}


// ****************************************************************************
// Create and Reset

void TitusSword1::create(Ogre::SceneManager* sentSceneMgr, Ogre::String theName)
{ // Creates the sword

  // Get a copy of the scene manager
  sceneMgr = sentSceneMgr;

  // Set the name of the weapon
  name = theName;

  // Set the wepon type
  this->setWeaponType(Sword);
		
  // Create the entity
  entity = sceneMgr->createEntity(name, "TidusSword1 02.mesh");
  meshBox = entity->getBoundingBox();
 
  // Attach the entity to the player node
  entityNode = sentSceneMgr->getRootSceneNode()->createChildSceneNode();
  entityNode->attachObject(entity);
		
 // Scale down size of player mesh	
  entityNode->setScale(scale,scale,scale); 

  // Set the position of the player node
  entityNode->setPosition(-100,-meshBox.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-400);
  length = meshBox.getSize().x;

  // Create the ray scene query
  raySceneQuery = sceneMgr->createRayQuery(Ogre::Ray());
  
  
  // Set query (for collisions) and visibility flags depending on whether active	
  if(active)
  {
	entity->setQueryFlags(true);
	entity->setVisibilityFlags(true);
  }
  else
  {
    entityNode->showBoundingBox(false);
	entity->setQueryFlags(false);
	entity->setVisibilityFlags(false);
  }
} // End Create

void TitusSword1::reset()
{ // Resets the weapon's flags and attributes

  entity->setQueryFlags(true);
  entity->setVisible(true);
  active = true;

  // Reset the position of the entity node
  entityNode->setPosition(-100,-meshBox.getCorner(Ogre::AxisAlignedBox::FAR_LEFT_BOTTOM).y*scale,-400);
} // End Reset

// ****************************************************************************